@@ -45,13 +45,6 @@ struct _GSettingsBackendPrivate
 {
   GSettingsBackendWatch *watches;
   GStaticMutex lock;
-  gchar *context;
-};
-
-enum
-{
-  PROP_0,
-  PROP_CONTEXT
 };
 
 /**
@@ -358,7 +351,7 @@ g_settings_backend_dispatch_signal (GSettingsBackend *backend,
 
   if (data1_free == NULL)
     data1_free = pointer_ignore;
- 
+
   context = g_settings_backend_get_active_context ();
   here_and_now = g_main_context_new ();
 
@@ -732,7 +725,8 @@ g_settings_backend_changed_tree (GSettingsBackend *backend,
  * g_settings_backend_read:
  * @backend: a #GSettingsBackend implementation
  * @key: the key to read
- * @expected_type: a #GVariantType hint
+ * @expected_type: a #GVariantType
+ * @default_value: if the default value should be returned
  * @returns: the value that was read, or %NULL
  *
  * Reads a key. This call will never block.
@@ -740,11 +734,13 @@ g_settings_backend_changed_tree (GSettingsBackend *backend,
  * If the key exists, the value associated with it will be returned.
  * If the key does not exist, %NULL will be returned.
  *
- * If @expected_type is given, it serves as a type hint to the backend.
- * If you expect a key of a certain type then you should give
- * @expected_type to increase your chances of getting it.  Some backends
- * may ignore this argument and return values of a different type; it is
- * mostly used by backends that don't store strong type information.
+ * The returned value will be of the type given in @expected_type.  If
+ * the backend stored a value of a different type then %NULL will be
+ * returned.
+ *
+ * If @default_value is %TRUE then this gets the default value from the
+ * backend (ie: the one that the backend would contain if
+ * g_settings_reset() were called).
  */
 GVariant *
 g_settings_backend_read (GSettingsBackend   *backend,
@@ -752,8 +748,18 @@ g_settings_backend_read (GSettingsBackend   *backend,
                          const GVariantType *expected_type,
                          gboolean            default_value)
 {
-  return G_SETTINGS_BACKEND_GET_CLASS (backend)
+  GVariant *value;
+
+  value = G_SETTINGS_BACKEND_GET_CLASS (backend)
     ->read (backend, key, expected_type, default_value);
+
+  if G_UNLIKELY (value && !g_variant_is_of_type (value, expected_type))
+    {
+      g_variant_unref (value);
+      value = NULL;
+    }
+
+  return value;
 }
 
 /*< private >
@@ -907,53 +913,12 @@ g_settings_backend_subscribe (GSettingsBackend *backend,
     ->subscribe (backend, name);
 }
 
-static void
-g_settings_backend_set_property (GObject         *object,
-                                 guint            prop_id,
-                                 const GValue    *value,
-                                 GParamSpec      *pspec)
-{
-  GSettingsBackend *backend = G_SETTINGS_BACKEND (object);
-
-  switch (prop_id)
-    {
-    case PROP_CONTEXT:
-      backend->priv->context = g_value_dup_string (value);
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
-static void
-g_settings_backend_get_property (GObject    *object,
-                                 guint       prop_id,
-                                 GValue     *value,
-                                 GParamSpec *pspec)
-{
-  GSettingsBackend *backend = G_SETTINGS_BACKEND (object);
-
-  switch (prop_id)
-    {
-    case PROP_CONTEXT:
-      g_value_set_string (value, backend->priv->context);
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
 static void
 g_settings_backend_finalize (GObject *object)
 {
   GSettingsBackend *backend = G_SETTINGS_BACKEND (object);
 
   g_static_mutex_unlock (&backend->priv->lock);
-  g_free (backend->priv->context);
 
   G_OBJECT_CLASS (g_settings_backend_parent_class)
     ->finalize (object);
@@ -982,32 +947,9 @@ g_settings_backend_class_init (GSettingsBackendClass *class)
   class->subscribe = ignore_subscription;
   class->unsubscribe = ignore_subscription;
 
-  gobject_class->get_property = g_settings_backend_get_property;
-  gobject_class->set_property = g_settings_backend_set_property;
   gobject_class->finalize = g_settings_backend_finalize;
 
   g_type_class_add_private (class, sizeof (GSettingsBackendPrivate));
-
-  /**
-   * GSettingsBackend:context:
-   *
-   * The "context" property gives a hint to the backend as to
-   * what storage to use. It is up to the implementation to make
-   * use of this information.
-   *
-   * E.g. DConf supports "user", "system", "defaults" and "login"
-   * contexts.
-   *
-   * If your backend supports different contexts, you should also
-   * provide an implementation of the supports_context() class
-   * function in #GSettingsBackendClass.
-   */
-  g_object_class_install_property (gobject_class, PROP_CONTEXT,
-    g_param_spec_string ("context", P_("Context"),
-                         P_("An identifier to decide which storage to use"),
-                         "", G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
-
 }
 
 /*< private >
@@ -1025,136 +967,61 @@ g_settings_backend_create_tree (void)
                           g_free, (GDestroyNotify) g_variant_unref);
 }
 
-
-static gpointer
-get_default_backend (const gchar *context)
-{
-  GIOExtension *extension = NULL;
-  GIOExtensionPoint *point;
-  GList *extensions;
-  const gchar *env;
-  GType type;
-
-  _g_io_modules_ensure_loaded ();
-
-  point = g_io_extension_point_lookup (G_SETTINGS_BACKEND_EXTENSION_POINT_NAME);
-
-  if ((env = getenv ("GSETTINGS_BACKEND")))
-    {
-      extension = g_io_extension_point_get_extension_by_name (point, env);
-
-      if (extension == NULL)
-        g_warning ("Can't find GSettings backend '%s' given in "
-                   "GSETTINGS_BACKEND environment variable", env);
-    }
-
-  if (extension == NULL)
-    {
-      extensions = g_io_extension_point_get_extensions (point);
-
-      if (extensions == NULL)
-        g_error ("No GSettingsBackend implementations exist.");
-
-      extension = extensions->data;
-    }
-
-  if (context[0] != '\0') /* (context != "") */
-    {
-      GSettingsBackendClass *backend_class;
-      GTypeClass *class;
-
-      class = g_io_extension_ref_class (extension);
-      backend_class = G_SETTINGS_BACKEND_CLASS (class);
-
-      if (backend_class->supports_context == NULL ||
-          !backend_class->supports_context (context))
-        {
-          g_type_class_unref (class);
-          return NULL;
-        }
-
-      g_type_class_unref (class);
-    }
-
-  type = g_io_extension_get_type (extension);
-
-  return g_object_new (type, "context", context, NULL);
-}
-
-static GHashTable *g_settings_backends;
-
 /*< private >
- * g_settings_backend_get_with_context:
- * @context: a context that might be used by the backend to determine
- *     which storage to use, or %NULL to use the default storage
+ * g_settings_backend_get_default:
  * @returns: the default #GSettingsBackend
  *
  * Returns the default #GSettingsBackend. It is possible to override
  * the default by setting the <envar>GSETTINGS_BACKEND</envar>
  * environment variable to the name of a settings backend.
  *
- * The @context parameter can be used to indicate that a different
- * than the default storage is desired. E.g. the DConf backend lets
- * you use "user", "system", "defaults" and "login" as contexts.
- *
- * If @context is not supported by the implementation, this function
- * returns an instance of the #GSettingsMemoryBackend.
- * See g_settings_backend_supports_context(),
- *
- * The user does not own the return value and it must not be freed.
+ * The user gets a reference to the backend.
  */
 GSettingsBackend *
-g_settings_backend_get_with_context (const gchar *context)
+g_settings_backend_get_default (void)
 {
-  GSettingsBackend *backend;
+  static gsize backend;
 
-  g_return_val_if_fail (context != NULL, NULL);
-
-  _g_io_modules_ensure_extension_points_registered ();
-
-  if (g_settings_backends == NULL)
-    g_settings_backends = g_hash_table_new (g_str_hash, g_str_equal);
-
-  backend = g_hash_table_lookup (g_settings_backends, context);
-
-  if (!backend)
+  if (g_once_init_enter (&backend))
     {
-      backend = get_default_backend (context);
+      GSettingsBackend *instance;
+      GIOExtensionPoint *point;
+      GIOExtension *extension;
+      GType extension_type;
+      GList *extensions;
+      const gchar *env;
 
-      if (!backend)
-        backend = g_null_settings_backend_new ();
+      _g_io_modules_ensure_loaded ();
 
-      g_hash_table_insert (g_settings_backends, g_strdup (context), backend);
+      point = g_io_extension_point_lookup (G_SETTINGS_BACKEND_EXTENSION_POINT_NAME);
+      extension = NULL;
+
+      if ((env = getenv ("GSETTINGS_BACKEND")))
+        {
+          extension = g_io_extension_point_get_extension_by_name (point, env);
+
+          if (extension == NULL)
+            g_warning ("Can't find GSettings backend '%s' given in "
+                       "GSETTINGS_BACKEND environment variable", env);
+        }
+
+      if (extension == NULL)
+        {
+          extensions = g_io_extension_point_get_extensions (point);
+
+          if (extensions == NULL)
+            g_error ("No GSettingsBackend implementations exist.");
+
+          extension = extensions->data;
+        }
+
+      extension_type = g_io_extension_get_type (extension);
+      instance = g_object_new (extension_type, NULL);
+
+      g_once_init_leave (&backend, (gsize) instance);
     }
 
-  return g_object_ref (backend);
-}
-
-/*< private >
- * g_settings_backend_supports_context:
- * @context: a context string that might be passed to
- *     g_settings_backend_new_with_context()
- * @returns: #TRUE if @context is supported
- *
- * Determines if the given context is supported by the default
- * GSettingsBackend implementation.
- */
-gboolean
-g_settings_backend_supports_context (const gchar *context)
-{
-  GSettingsBackend *backend;
-
-  g_return_val_if_fail (context != NULL, FALSE);
-
-  backend = get_default_backend (context);
-
-  if (backend)
-    {
-      g_object_unref (backend);
-      return TRUE;
-    }
-
-  return FALSE;
+  return g_object_ref ((void *) backend);
 }
 
 /*< private >
@@ -1181,42 +1048,22 @@ g_settings_backend_get_permission (GSettingsBackend *backend,
   return g_simple_permission_new (TRUE);
 }
 
-/**
- * g_settings_backend_setup:
- * @context: a context string (not %NULL or "")
- * @backend: a #GSettingsBackend
+/*< private >
+ * g_settings_backend_sync_default:
  *
- * Sets up @backend for use with #GSettings.
- *
- * If you create a #GSettings with its context property set to @context
- * then it will use the backend given to this function.  See
- * g_settings_new_with_context().
- *
- * The backend must be set up before any settings objects are created
- * for the named context.
- *
- * It is not possible to specify a backend for the default context.
- *
- * This function takes a reference on @backend and never releases it.
- *
- * Since: 2.26
- **/
+ * Syncs the default backend.
+ */
 void
-g_settings_backend_setup (const gchar      *context,
-                          GSettingsBackend *backend)
+g_settings_backend_sync_default (void)
 {
-  g_return_if_fail (context[0] != '\0');
-  g_return_if_fail (G_IS_SETTINGS_BACKEND (backend));
+  GSettingsBackendClass *class;
+  GSettingsBackend *backend;
 
-  if (g_settings_backends == NULL)
-    g_settings_backends = g_hash_table_new (g_str_hash, g_str_equal);
+  backend = g_settings_backend_get_default ();
+  class = G_SETTINGS_BACKEND_GET_CLASS (backend);
 
-  if (g_hash_table_lookup (g_settings_backends, context))
-    g_error ("A GSettingsBackend already exists for context '%s'", context);
-
-  g_hash_table_insert (g_settings_backends,
-                       g_strdup (context),
-                       g_object_ref (backend));
+  if (class->sync)
+    class->sync (backend);
 }
 
 #define __G_SETTINGS_BACKEND_C__
