@@ -1,5 +1,6 @@
 /* GObject - GLib Type, Object, Parameter and Signal Library
  * Copyright (C) 1997-1999, 2000-2001 Tim Janik and Red Hat, Inc.
+ * Copyright © 2010 Christian Persch
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,18 +22,18 @@
  * MT safe
  */
 
-#include	"gvaluetypes.h"
+#include "config.h"
 
-#include	"gvaluecollector.h"
+#include <string.h>
+#include <stdlib.h> /* qsort() */
 
+#include "gvaluetypes.h"
+#include "gvaluecollector.h"
 #include "gobject.h"
 #include "gparam.h"
 #include "gboxed.h"
 #include "genums.h"
-
-#include	"gobjectalias.h"
-#include	<string.h>
-#include	<stdlib.h>	/* qsort() */
+#include "gobjectalias.h"
 
 
 /* --- value functions --- */
@@ -361,6 +362,63 @@ value_lcopy_pointer (const GValue *value,
   return NULL;
 }
 
+static void
+value_free_variant (GValue *value)
+{
+  if (!(value->data[1].v_uint & G_VALUE_NOCOPY_CONTENTS) &&
+      value->data[0].v_pointer)
+    g_variant_unref (value->data[0].v_pointer);
+}
+
+static void
+value_copy_variant (const GValue *src_value,
+		   GValue	*dest_value)
+{
+  if (src_value->data[0].v_pointer)
+    dest_value->data[0].v_pointer = g_variant_ref_sink (src_value->data[0].v_pointer);
+  else
+    dest_value->data[0].v_pointer = NULL;
+}
+
+static gchar*
+value_collect_variant (GValue	  *value,
+		      guint        n_collect_values,
+		      GTypeCValue *collect_values,
+		      guint        collect_flags)
+{
+  if (!collect_values[0].v_pointer)
+    value->data[0].v_pointer = NULL;
+  else if (collect_flags & G_VALUE_NOCOPY_CONTENTS)
+    {
+      value->data[0].v_pointer = collect_values[0].v_pointer;
+      value->data[1].v_uint = G_VALUE_NOCOPY_CONTENTS;
+    }
+  else
+    value->data[0].v_pointer = g_variant_ref_sink (collect_values[0].v_pointer);
+
+  return NULL;
+}
+
+static gchar*
+value_lcopy_variant (const GValue *value,
+		    guint         n_collect_values,
+		    GTypeCValue  *collect_values,
+		    guint         collect_flags)
+{
+  GVariant **variant_p = collect_values[0].v_pointer;
+
+  if (!variant_p)
+    return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
+
+  if (!value->data[0].v_pointer)
+    *variant_p = NULL;
+  else if (collect_flags & G_VALUE_NOCOPY_CONTENTS)
+    *variant_p = value->data[0].v_pointer;
+  else
+    *variant_p = g_variant_ref_sink (value->data[0].v_pointer);
+
+  return NULL;
+}
 
 /* --- type initialization --- */
 void
@@ -550,10 +608,35 @@ g_value_types_init (void)
     type = g_type_register_fundamental (G_TYPE_POINTER, g_intern_static_string ("gpointer"), &info, &finfo, 0);
     g_assert (type == G_TYPE_POINTER);
   }
+
+  /* G_TYPE_VARIANT
+   */
+  {
+    static const GTypeValueTable value_table = {
+      value_init_pointer,	/* value_init */
+      value_free_variant,	/* value_free */
+      value_copy_variant,	/* value_copy */
+      value_peek_pointer0,	/* value_peek_pointer */
+      "p",			/* collect_format */
+      value_collect_variant,	/* collect_value */
+      "p",			/* lcopy_format */
+      value_lcopy_variant,	/* lcopy_value */
+    };
+    info.value_table = &value_table;
+    type = g_type_register_fundamental (G_TYPE_VARIANT, g_intern_static_string ("GVariant"), &info, &finfo, 0);
+    g_assert (type == G_TYPE_VARIANT);
+  }
 }
 
 
 /* --- GValue functions --- */
+/**
+ * g_value_set_char:
+ * @value: a valid #GValue of type %G_TYPE_CHAR
+ * @v_char: character value to be set
+ *
+ * Set the contents of a %G_TYPE_CHAR #GValue to @v_char.
+ */
 void
 g_value_set_char (GValue *value,
 		  gchar	  v_char)
@@ -563,6 +646,14 @@ g_value_set_char (GValue *value,
   value->data[0].v_int = v_char;
 }
 
+/**
+ * g_value_get_char:
+ * @value: a valid #GValue of type %G_TYPE_CHAR
+ *
+ * Get the contents of a %G_TYPE_CHAR #GValue.
+ * 
+ * Returns: character contents of @value
+ */
 gchar
 g_value_get_char (const GValue *value)
 {
@@ -571,6 +662,13 @@ g_value_get_char (const GValue *value)
   return value->data[0].v_int;
 }
 
+/**
+ * g_value_set_uchar:
+ * @value: a valid #GValue of type %G_TYPE_UCHAR
+ * @v_uchar: unsigned character value to be set
+ *
+ * Set the contents of a %G_TYPE_UCHAR #GValue to @v_uchar.
+ */
 void
 g_value_set_uchar (GValue *value,
 		   guchar  v_uchar)
@@ -580,6 +678,14 @@ g_value_set_uchar (GValue *value,
   value->data[0].v_uint = v_uchar;
 }
 
+/**
+ * g_value_get_uchar:
+ * @value: a valid #GValue of type %G_TYPE_UCHAR
+ *
+ * Get the contents of a %G_TYPE_UCHAR #GValue.
+ *
+ * Returns: unsigned character contents of @value
+ */
 guchar
 g_value_get_uchar (const GValue *value)
 {
@@ -588,6 +694,13 @@ g_value_get_uchar (const GValue *value)
   return value->data[0].v_uint;
 }
 
+/**
+ * g_value_set_boolean:
+ * @value: a valid #GValue of type %G_TYPE_BOOLEAN
+ * @v_boolean: boolean value to be set
+ *
+ * Set the contents of a %G_TYPE_BOOLEAN #GValue to @v_boolean.
+ */
 void
 g_value_set_boolean (GValue  *value,
 		     gboolean v_boolean)
@@ -597,6 +710,14 @@ g_value_set_boolean (GValue  *value,
   value->data[0].v_int = v_boolean != FALSE;
 }
 
+/**
+ * g_value_get_boolean:
+ * @value: a valid #GValue of type %G_TYPE_BOOLEAN
+ *
+ * Get the contents of a %G_TYPE_BOOLEAN #GValue.
+ *
+ * Returns: boolean contents of @value
+ */
 gboolean
 g_value_get_boolean (const GValue *value)
 {
@@ -605,6 +726,13 @@ g_value_get_boolean (const GValue *value)
   return value->data[0].v_int;
 }
 
+/**
+ * g_value_set_int:
+ * @value: a valid #GValue of type %G_TYPE_INT
+ * @v_int: integer value to be set
+ *
+ * Set the contents of a %G_TYPE_INT #GValue to @v_int.
+ */
 void
 g_value_set_int (GValue *value,
 		 gint	 v_int)
@@ -614,6 +742,14 @@ g_value_set_int (GValue *value,
   value->data[0].v_int = v_int;
 }
 
+/**
+ * g_value_get_int:
+ * @value: a valid #GValue of type %G_TYPE_INT
+ *
+ * Get the contents of a %G_TYPE_INT #GValue.
+ *
+ * Returns: integer contents of @value
+ */
 gint
 g_value_get_int (const GValue *value)
 {
@@ -622,6 +758,13 @@ g_value_get_int (const GValue *value)
   return value->data[0].v_int;
 }
 
+/**
+ * g_value_set_uint:
+ * @value: a valid #GValue of type %G_TYPE_UINT
+ * @v_uint: unsigned integer value to be set
+ *
+ * Set the contents of a %G_TYPE_UINT #GValue to @v_uint.
+ */
 void
 g_value_set_uint (GValue *value,
 		  guint	  v_uint)
@@ -631,6 +774,14 @@ g_value_set_uint (GValue *value,
   value->data[0].v_uint = v_uint;
 }
 
+/**
+ * g_value_get_uint:
+ * @value: a valid #GValue of type %G_TYPE_UINT
+ *
+ * Get the contents of a %G_TYPE_UINT #GValue.
+ *
+ * Returns: unsigned integer contents of @value
+ */
 guint
 g_value_get_uint (const GValue *value)
 {
@@ -639,6 +790,13 @@ g_value_get_uint (const GValue *value)
   return value->data[0].v_uint;
 }
 
+/**
+ * g_value_set_long:
+ * @value: a valid #GValue of type %G_TYPE_LONG
+ * @v_long: long integer value to be set
+ *
+ * Set the contents of a %G_TYPE_LONG #GValue to @v_long.
+ */
 void
 g_value_set_long (GValue *value,
 		  glong	  v_long)
@@ -648,6 +806,14 @@ g_value_set_long (GValue *value,
   value->data[0].v_long = v_long;
 }
 
+/**
+ * g_value_get_long:
+ * @value: a valid #GValue of type %G_TYPE_LONG
+ *
+ * Get the contents of a %G_TYPE_LONG #GValue.
+ *
+ * Returns: long integer contents of @value
+ */
 glong
 g_value_get_long (const GValue *value)
 {
@@ -656,6 +822,13 @@ g_value_get_long (const GValue *value)
   return value->data[0].v_long;
 }
 
+/**
+ * g_value_set_ulong:
+ * @value: a valid #GValue of type %G_TYPE_ULONG
+ * @v_ulong: unsigned long integer value to be set
+ *
+ * Set the contents of a %G_TYPE_ULONG #GValue to @v_ulong.
+ */
 void
 g_value_set_ulong (GValue *value,
 		   gulong  v_ulong)
@@ -665,6 +838,14 @@ g_value_set_ulong (GValue *value,
   value->data[0].v_ulong = v_ulong;
 }
 
+/**
+ * g_value_get_ulong:
+ * @value: a valid #GValue of type %G_TYPE_ULONG
+ *
+ * Get the contents of a %G_TYPE_ULONG #GValue.
+ *
+ * Returns: unsigned long integer contents of @value
+ */
 gulong
 g_value_get_ulong (const GValue *value)
 {
@@ -673,6 +854,14 @@ g_value_get_ulong (const GValue *value)
   return value->data[0].v_ulong;
 }
 
+/**
+ * g_value_get_int64:
+ * @value: a valid #GValue of type %G_TYPE_INT64
+ *
+ * Get the contents of a %G_TYPE_INT64 #GValue.
+ *
+ * Returns: 64bit integer contents of @value
+ */
 void
 g_value_set_int64 (GValue *value,
 		   gint64  v_int64)
@@ -682,6 +871,13 @@ g_value_set_int64 (GValue *value,
   value->data[0].v_int64 = v_int64;
 }
 
+/**
+ * g_value_set_int64:
+ * @value: a valid #GValue of type %G_TYPE_INT64
+ * @v_int64: 64bit integer value to be set
+ *
+ * Set the contents of a %G_TYPE_INT64 #GValue to @v_int64.
+ */
 gint64
 g_value_get_int64 (const GValue *value)
 {
@@ -690,6 +886,13 @@ g_value_get_int64 (const GValue *value)
   return value->data[0].v_int64;
 }
 
+/**
+ * g_value_set_uint64:
+ * @value: a valid #GValue of type %G_TYPE_UINT64
+ * @v_uint64: unsigned 64bit integer value to be set
+ *
+ * Set the contents of a %G_TYPE_UINT64 #GValue to @v_uint64.
+ */
 void
 g_value_set_uint64 (GValue *value,
 		    guint64 v_uint64)
@@ -699,6 +902,14 @@ g_value_set_uint64 (GValue *value,
   value->data[0].v_uint64 = v_uint64;
 }
 
+/**
+ * g_value_get_uint64:
+ * @value: a valid #GValue of type %G_TYPE_UINT64
+ *
+ * Get the contents of a %G_TYPE_UINT64 #GValue.
+ *
+ * Returns: unsigned 64bit integer contents of @value
+ */
 guint64
 g_value_get_uint64 (const GValue *value)
 {
@@ -707,6 +918,13 @@ g_value_get_uint64 (const GValue *value)
   return value->data[0].v_uint64;
 }
 
+/**
+ * g_value_set_float:
+ * @value: a valid #GValue of type %G_TYPE_FLOAT
+ * @v_float: float value to be set
+ *
+ * Set the contents of a %G_TYPE_FLOAT #GValue to @v_float.
+ */
 void
 g_value_set_float (GValue *value,
 		   gfloat  v_float)
@@ -716,6 +934,14 @@ g_value_set_float (GValue *value,
   value->data[0].v_float = v_float;
 }
 
+/**
+ * g_value_get_float:
+ * @value: a valid #GValue of type %G_TYPE_FLOAT
+ *
+ * Get the contents of a %G_TYPE_FLOAT #GValue.
+ *
+ * Returns: float contents of @value
+ */
 gfloat
 g_value_get_float (const GValue *value)
 {
@@ -724,6 +950,13 @@ g_value_get_float (const GValue *value)
   return value->data[0].v_float;
 }
 
+/**
+ * g_value_set_double:
+ * @value: a valid #GValue of type %G_TYPE_DOUBLE
+ * @v_double: double value to be set
+ *
+ * Set the contents of a %G_TYPE_DOUBLE #GValue to @v_double.
+ */
 void
 g_value_set_double (GValue *value,
 		    gdouble v_double)
@@ -733,6 +966,14 @@ g_value_set_double (GValue *value,
   value->data[0].v_double = v_double;
 }
 
+/**
+ * g_value_get_double:
+ * @value: a valid #GValue of type %G_TYPE_DOUBLE
+ *
+ * Get the contents of a %G_TYPE_DOUBLE #GValue.
+ *
+ * Returns: double contents of @value
+ */
 gdouble
 g_value_get_double (const GValue *value)
 {
@@ -741,6 +982,13 @@ g_value_get_double (const GValue *value)
   return value->data[0].v_double;
 }
 
+/**
+ * g_value_set_string:
+ * @value: a valid #GValue of type %G_TYPE_STRING
+ * @v_string: caller-owned string to be duplicated for the #GValue
+ *
+ * Set the contents of a %G_TYPE_STRING #GValue to @v_string.
+ */
 void
 g_value_set_string (GValue	*value,
 		    const gchar *v_string)
@@ -759,6 +1007,15 @@ g_value_set_string (GValue	*value,
   value->data[0].v_pointer = new_val;
 }
 
+/**
+ * g_value_set_static_string:
+ * @value: a valid #GValue of type %G_TYPE_STRING
+ * @v_string: static string to be set
+ *
+ * Set the contents of a %G_TYPE_STRING #GValue to @v_string.
+ * The string is assumed to be static, and is thus not duplicated
+ * when setting the #GValue.
+ */
 void
 g_value_set_static_string (GValue      *value,
 			   const gchar *v_string)
@@ -771,6 +1028,15 @@ g_value_set_static_string (GValue      *value,
   value->data[0].v_pointer = (gchar*) v_string;
 }
 
+/**
+ * g_value_set_string_take_ownership:
+ * @value: a valid #GValue of type %G_TYPE_STRING
+ * @v_string: duplicated unowned string to be set
+ *
+ * This is an internal function introduced mainly for C marshallers.
+ *
+ * Deprecated: 2.4: Use g_value_take_string() instead.
+ */
 void
 g_value_set_string_take_ownership (GValue *value,
 				   gchar  *v_string)
@@ -778,6 +1044,15 @@ g_value_set_string_take_ownership (GValue *value,
   g_value_take_string (value, v_string);
 }
 
+/**
+ * g_value_take_string:
+ * @value: a valid #GValue of type %G_TYPE_STRING
+ * @v_string: string to take ownership of
+ *
+ * Sets the contents of a %G_TYPE_STRING #GValue to @v_string.
+ *
+ * Since: 2.4
+ */
 void
 g_value_take_string (GValue *value,
 		     gchar  *v_string)
@@ -791,6 +1066,14 @@ g_value_take_string (GValue *value,
   value->data[0].v_pointer = v_string;
 }
 
+/**
+ * g_value_get_string:
+ * @value: a valid #GValue of type %G_TYPE_STRING
+ *
+ * Get the contents of a %G_TYPE_STRING #GValue.
+ *
+ * Returns: string content of @value
+ */
 G_CONST_RETURN gchar*
 g_value_get_string (const GValue *value)
 {
@@ -799,6 +1082,14 @@ g_value_get_string (const GValue *value)
   return value->data[0].v_pointer;
 }
 
+/**
+ * g_value_dup_string:
+ * @value: a valid #GValue of type %G_TYPE_STRING
+ *
+ * Get a copy the contents of a %G_TYPE_STRING #GValue.
+ *
+ * Returns: a newly allocated copy of the string content of @value
+ */
 gchar*
 g_value_dup_string (const GValue *value)
 {
@@ -807,6 +1098,13 @@ g_value_dup_string (const GValue *value)
   return g_strdup (value->data[0].v_pointer);
 }
 
+/**
+ * g_value_set_pointer:
+ * @value: a valid #GValue of %G_TYPE_POINTER
+ * @v_pointer: pointer value to be set
+ *
+ * Set the contents of a pointer #GValue to @v_pointer.
+ */
 void
 g_value_set_pointer (GValue  *value,
 		     gpointer v_pointer)
@@ -816,6 +1114,14 @@ g_value_set_pointer (GValue  *value,
   value->data[0].v_pointer = v_pointer;
 }
 
+/**
+ * g_value_get_pointer:
+ * @value: a valid #GValue of %G_TYPE_POINTER
+ *
+ * Get the contents of a pointer #GValue.
+ *
+ * Returns: pointer contents of @value
+ */
 gpointer
 g_value_get_pointer (const GValue *value)
 {
@@ -834,7 +1140,16 @@ g_gtype_get_type (void)
   return type;
 }
 
-void 
+/**
+ * g_value_set_gtype:
+ * @value: a valid #GValue of type %G_TYPE_GTYPE
+ * @v_gtype: #GType to be set
+ *
+ * Set the contents of a %G_TYPE_GTYPE #GValue to @v_gtype.
+ *
+ * Since: 2.12
+ */
+void
 g_value_set_gtype (GValue *value,
 		   GType   v_gtype)
 {
@@ -844,7 +1159,17 @@ g_value_set_gtype (GValue *value,
   
 }
 
-GType	              
+/**
+ * g_value_get_gtype:
+ * @value: a valid #GValue of type %G_TYPE_GTYPE
+ *
+ * Get the contents of a %G_TYPE_GTYPE #GValue.
+ *
+ * Since: 2.12
+ *
+ * Returns: the #GType stored in @value
+ */
+GType
 g_value_get_gtype (const GValue *value)
 {
   g_return_val_if_fail (G_VALUE_HOLDS_GTYPE (value), 0);
@@ -852,7 +1177,89 @@ g_value_get_gtype (const GValue *value)
   return value->data[0].v_long;
 }
 
+/**
+ * g_value_set_variant:
+ * @value: a valid #GValue of %G_TYPE_VARIANT
+ * @variant: a #GVariant, or %NULL
+ *
+ * Set the contents of a variant #GValue to @variant.
+ * If the variant is floating, it is consumed.
+ *
+ * Since: 2.26
+ */
+void
+g_value_set_variant (GValue   *value,
+                     GVariant *variant)
+{
+  GVariant *old_variant;
 
+  g_return_if_fail (G_VALUE_HOLDS_VARIANT (value));
+
+  old_variant = value->data[0].v_pointer;
+
+  if (variant)
+    value->data[0].v_pointer = g_variant_ref_sink (variant);
+  else
+    value->data[0].v_pointer = NULL;
+
+  if (old_variant)
+    g_variant_unref (old_variant);
+}
+
+/**
+ * g_value_get_variant:
+ * @value: a valid #GValue of %G_TYPE_VARIANT
+ *
+ * Get the contents of a variant #GValue.
+ *
+ * Returns: variant contents of @value
+ *
+ * Since: 2.26
+ */
+GVariant*
+g_value_get_variant (const GValue *value)
+{
+  g_return_val_if_fail (G_VALUE_HOLDS_VARIANT (value), NULL);
+
+  return value->data[0].v_pointer;
+}
+
+/**
+ * g_value_dup_variant:
+ * @value: a valid #GValue of %G_TYPE_VARIANT
+ *
+ * Get the contents of a variant #GValue, increasing its refcount.
+ *
+ * Returns: variant contents of @value, should be unrefed using
+ *   g_variant_unref() when no longer needed
+ *
+ * Since: 2.26
+ */
+GVariant*
+g_value_dup_variant (const GValue *value)
+{
+  GVariant *variant;
+
+  g_return_val_if_fail (G_VALUE_HOLDS_VARIANT (value), NULL);
+
+  variant = value->data[0].v_pointer;
+  if (variant)
+    g_variant_ref_sink (variant);
+
+  return variant;
+}
+
+/**
+ * g_strdup_value_contents:
+ * @value: #GValue which contents are to be described.
+ *
+ * Return a newly allocated string, which describes the contents of a
+ * #GValue.  The main purpose of this function is to describe #GValue
+ * contents for debugging output, the way in which the contents are
+ * described may change between different GLib versions.
+ *
+ * Returns: Newly allocated string.
+ */
 gchar*
 g_strdup_value_contents (const GValue *value)
 {
@@ -915,6 +1322,15 @@ g_strdup_value_contents (const GValue *value)
   return contents;
 }
 
+/**
+ * g_pointer_type_register_static:
+ * @name: the name of the new pointer type.
+ *
+ * Creates a new %G_TYPE_POINTER derived type id for a new
+ * pointer type with name @name.
+ *
+ * Returns: a new %G_TYPE_POINTER derived type id for @name.
+ */
 GType
 g_pointer_type_register_static (const gchar *name)
 {
